"""
figur.zenodo
~~~~~~~~~~~~

This module implements the API to Zenodo.
"""

import requests
import wget


def download(doi, filepath):
    url = f"https://doi.org/{doi}"
    r = requests.get(url)
    record = r.url.split("/")[-1].strip()
    url = f"https://zenodo.org/api/records/{record}"
    r = requests.get(url)
    if r.ok:
        print("Downloading model from Zenodo...")
        print(f"Target directory: {filepath}")
        response = r.json()
        files = response["files"]
        total = sum(file["size"] for file in files)
        for file in files:
            link = file["links"]["self"]
            size = file["size"] / 2 ** 20
            print(f"Total size: {size:.1f} MB")
            fname = file["key"]
            checksum = file["checksum"]
            filename = wget.download(link, filepath)
            return filename
    else:
        raise Excpetion("Unable to download model from Zenodo.")



"""
figur.api
~~~~~~~~~

This module implements the high-level API.
"""

import pandas as pd

from . import model
from . import utils


FILENAME = utils.RESOURCE["filename"]
PATH = utils.cached(FILENAME)
TAGGER = model.Model(PATH)


def tag(text: str) -> pd.DataFrame:
    """Tag named entities in a given text.
    """
    tagged = utils.process(text, TAGGER)
    return pd.DataFrame(tagged)



"""
figur.model
~~~~~~~~~~~

This module implements the NER model class.
"""

from pathlib import Path

from flair.models import SequenceTagger
from flair.data import Sentence


class Model:
    def __init__(self, path: Path):
        self.path = path
        self.tagger = SequenceTagger.load_from_file(path)

    def predict(self, sentence: str):
        """Predict labels for a given sentence.
        """
        s = Sentence(sentence)
        self.tagger.predict(s)
        for token in s.tokens:
            tag = token.get_tag("ner").value
            word = token.text
            yield word.strip(), tag if tag != "word" else "_"



"""
figur.utils
~~~~~~~~~~~

This module implements basic helper functions.
"""

import urllib
from pathlib import Path

from spacy.lang.de import German

from . import zenodo


HOME = Path.home()
CACHE_DIR = Path("models")
CACHE_ROOT = Path(HOME, ".figur")
RESOURCE = {"filename": "figurenerkennung-0.0.1.pt",
            "doi": "10.5281/zenodo.2592325"}


def cached(path: str) -> Path:
    cache = Path(CACHE_ROOT, CACHE_DIR, path)
    if cache.exists():
        return cache
    else:
        return _get_from_cache(RESOURCE["doi"], cache)


def _get_from_cache(doi: str, cache: Path) -> Path:
    if not cache.parent.exists():
        cache.parent.mkdir(parents=True)
    return zenodo.download(doi, str(cache))


def segment(text: str):
    nlp = German()
    nlp.add_pipe(nlp.create_pipe("sentencizer"))
    document = nlp(text)
    for sentence in document.sents:
        yield sentence.text


def process(text: str, model):
    tagged = {"SentenceId": list(),
              "Token": list(),
              "Tag": list()}
    for n, sentence in enumerate(segment(text)):
        prediction = model.predict(sentence)
        for word, tag in prediction:
            tagged["SentenceId"].append(n)
            tagged["Token"].append(word)
            tagged["Tag"].append(tag.title() if tag == "pron" else tag)
    return tagged



"""
Figurenerkennung for German literary texts
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

`figur` is very easy to use:

```
>>> import figur
>>> text = "Der Gärtner entfernte sich eilig, und Eduard folgte bald."
>>> figur.tag(text)
   SentenceId      Token      Tag
0           0        Der        _
1           0    Gärtner  AppTdfW
2           0  entfernte        _
3           0       sich     Pron
4           0     eilig,        _
5           0        und        _
6           0     Eduard     Core
7           0     folgte        _
8           0      bald.        _
```
"""

from .api import tag



import io
import os
import shutil
import sys
import setuptools


NAME = "figur"
DESCRIPTION = "Figurenerkennung for German literary texts"
URL = "https://github.com/severinsimmler/figur"
AUTHOR = "Severin Simmler"
REQUIRES_PYTHON = ">=3.6.0"
VERSION = "0.0.8"
REQUIRED = ["flair>=0.4.1",
            "spacy>=2.0.18",
            "pandas>=0.24.1",
            "wget>=3.2",
            "requests>=2.21.0",
            # pytorch-pretrained-bret v0.6.2 is somehow broken...
            "pytorch-pretrained-bert<=0.6.1"]


with io.open("README.md", encoding="utf-8") as readme:
    long_description = f"\n{readme.read()}"


class UploadCommand(setuptools.Command):
    description = "Build and publish the package."
    user_options = list()

    @staticmethod
    def status(s):
        print(f"\033[1m{s}\033[0m")

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds...")
            shutil.rmtree("dist")
        except OSError:
            pass

        self.status("Building source and wheel distribution...")
        os.system(f"{sys.executable} setup.py sdist bdist_wheel")

        self.status("Uploading the package to PyPI via Twine...")
        os.system("twine upload dist/*")

        self.status("Pushing git tags...")
        os.system(f"git tag v{VERSION}")
        os.system("git push --tags")

        sys.exit()


setuptools.setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=setuptools.find_packages(exclude=("tests",)),
    install_requires=REQUIRED,
    include_package_data=True,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy"
    ],
    cmdclass={
        "upload": UploadCommand,
    },
)
